@@ -1,19 +1,26 @@
 // SPDX-License-Identifier: Apache-2.0
 // Copyright © 2021 Will Ross
+extern crate alloc;
+
+use alloc::collections::VecDeque;
+use alloc::rc::Rc;
+use core::cell::{Ref, RefCell};
 use core::convert::TryInto;
 use core::ops::RangeInclusive;
 
 use arrayvec::ArrayVec;
 use embedded_hal::blocking::i2c;
 
-use super::eeprom_data::{mlx90640_eeprom_data, mlx90641_eeprom_data, EEPROM_LENGTH};
-use crate::{mlx90640, mlx90641, Address};
+use super::eeprom_data::{mlx90640_datasheet_eeprom, mlx90641_datasheet_eeprom, EEPROM_LENGTH};
+use super::mlx90640_example_data;
+use crate::{mlx90640, mlx90641, Address, MelexisCamera};
 
 /// The number of bytes the MLX90640 has of RAM.
 ///
-/// The MLX90640 has its RAM from 0x0400 through 0x07FF, representing 768 pixels along with 64
-/// other addresses (half of which are reserved). Each address corresponds to 16 bits of data.
-pub(crate) const MLX90640_RAM_LENGTH: usize = (0x0800 - 0x0400) * 2;
+/// The MLX90640 has its RAM from 0x0400 through 0x07FF, but only uses up through 0x073F. The used
+/// range representing 768 pixels along with 64 other addresses (half of which are reserved). Each
+/// address corresponds to 16 bits of data.
+pub(crate) const MLX90640_RAM_LENGTH: usize = (0x0740 - 0x0400) * 2;
 
 /// The number of bytes the MLX90641 has of RAM.
 ///
@@ -41,6 +48,8 @@ const I2C_CONFIG_REGISTER_ADDRESS: u16 = 0x800F;
 // Only the last four bits of the I2C config register are documented.
 const I2C_CONFIG_REGISTER_WRITE_MASK: [u8; 2] = [0x00, 0x0F];
 
+const RECENT_OPERATIONS_QUEUE_LENGTH: usize = 32;
+
 #[allow(dead_code)]
 #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord)]
 #[repr(u16)]
@@ -79,6 +88,12 @@ pub(crate) enum MockError {
     IllegalOperation,
 }
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub(crate) enum I2cOperation {
+    Write { address: Address, length: usize },
+    Read { address: Address, length: usize },
+}
+
 #[derive(Clone, Debug)]
 pub(crate) struct MockCameraBus<const RAM_LENGTH: usize> {
     i2c_address: u8,
@@ -86,11 +101,47 @@ pub(crate) struct MockCameraBus<const RAM_LENGTH: usize> {
     ram_range: RangeInclusive<u16>,
     eeprom_range: RangeInclusive<u16>,
     register_range: RangeInclusive<u16>,
-    eeprom_data: [u8; EEPROM_LENGTH],
-    ram_data: [u8; RAM_LENGTH],
-    status_register: [u8; 2],
-    control_register: [u8; 2],
-    i2c_config_register: [u8; 2],
+    eeprom_data: Rc<RefCell<[u8; EEPROM_LENGTH]>>,
+    ram_data: Rc<RefCell<[u8; RAM_LENGTH]>>,
+    status_register: Rc<RefCell<[u8; 2]>>,
+    control_register: Rc<RefCell<[u8; 2]>>,
+    i2c_config_register: Rc<RefCell<[u8; 2]>>,
+    recent_operations: Rc<RefCell<VecDeque<I2cOperation>>>,
+}
+
+impl MockCameraBus<MLX90640_RAM_LENGTH> {
+    pub(crate) fn new_mlx90640(
+        i2c_address: u8,
+        eeprom: &[u8],
+        ram: &[u8],
+        control_register: &[u8],
+        status_register: &[u8],
+        i2c_register: &[u8],
+    ) -> Self {
+        let mut eeprom_data = [0u8; mlx90640_example_data::EEPROM_DATA.len()];
+        eeprom_data.copy_from_slice(eeprom);
+        let mut ram_data = [0u8; MLX90640_RAM_LENGTH];
+        ram_data.copy_from_slice(ram);
+        let mut control_register_owned = [0u8; 2];
+        control_register_owned.copy_from_slice(control_register);
+        let mut status_register_owned = [0u8; 2];
+        status_register_owned.copy_from_slice(status_register);
+        let mut i2c_register_owned = [0u8; 2];
+        i2c_register_owned.copy_from_slice(i2c_register);
+        MockCameraBus {
+            i2c_address,
+            rom_range: 0x0000..=0x03FF,
+            ram_range: (mlx90640::RamAddress::Base.into())..=(mlx90640::RamAddress::End.into()),
+            eeprom_range: 0x2400..=0x273F,
+            register_range: 0x8000..=0x8016,
+            eeprom_data: Rc::new(RefCell::new(eeprom_data)),
+            ram_data: Rc::new(RefCell::new(ram_data)),
+            status_register: Rc::new(RefCell::new(status_register_owned)),
+            control_register: Rc::new(RefCell::new(control_register_owned)),
+            i2c_config_register: Rc::new(RefCell::new(i2c_register_owned)),
+            recent_operations: Rc::new(RefCell::new(VecDeque::new())),
+        }
+    }
 }
 
 impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
@@ -110,7 +161,7 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
     }
 
     /// Access a slice containing `count` 16-bit words, starting at the given address.
-    pub(crate) fn get(&self, address: Address, byte_count: usize) -> Result<&[u8], MockError> {
+    pub(crate) fn get(&self, address: Address, byte_count: usize) -> Result<Ref<[u8]>, MockError> {
         let start_address: u16 = address.into();
         let end_address = start_address + (byte_count / 2) as u16;
         // classify the address and check if it's supposed to be read
@@ -126,7 +177,9 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
             if self.ram_range.contains(&(end_address - 1)) {
                 let slice_start = (start_address - self.ram_range.start()) as usize * 2;
                 let slice_end = slice_start + byte_count;
-                Ok(&self.ram_data[slice_start..slice_end])
+                let slice_ref = Ref::map(self.ram_data.borrow(), |r| &r[slice_start..slice_end]);
+                Ok(slice_ref)
+                //Ok(&self.ram_data[slice_start..slice_end])
             } else {
                 // Illegal, but use the end address as the given address.
                 Err(MockError::IllegalAccess(end_address.into()))
@@ -136,7 +189,8 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
             if self.eeprom_range.contains(&(end_address - 1)) {
                 let slice_start = (start_address - self.eeprom_range.start()) as usize * 2;
                 let slice_end = slice_start + byte_count;
-                Ok(&self.eeprom_data[slice_start..slice_end])
+                let slice_ref = Ref::map(self.eeprom_data.borrow(), |r| &r[slice_start..slice_end]);
+                Ok(slice_ref)
             } else {
                 Err(MockError::IllegalAccess(end_address.into()))
             }
@@ -147,9 +201,18 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
                 Err(MockError::IllegalAccess(end_address.into()))
             } else {
                 match start_address {
-                    STATUS_REGISTER_ADDRESS => Ok(&self.status_register[..]),
-                    CONTROL_REGISTER_ONE_ADDRESS => Ok(&self.control_register[..]),
-                    I2C_CONFIG_REGISTER_ADDRESS => Ok(&self.i2c_config_register[..]),
+                    STATUS_REGISTER_ADDRESS => {
+                        let slice_ref = Ref::map(self.status_register.borrow(), |r| &r[..]);
+                        Ok(slice_ref)
+                    }
+                    CONTROL_REGISTER_ONE_ADDRESS => {
+                        let slice_ref = Ref::map(self.control_register.borrow(), |r| &r[..]);
+                        Ok(slice_ref)
+                    }
+                    I2C_CONFIG_REGISTER_ADDRESS => {
+                        let slice_ref = Ref::map(self.i2c_config_register.borrow(), |r| &r[..]);
+                        Ok(slice_ref)
+                    }
                     _ => Err(MockError::IllegalAccess(address)),
                 }
             }
@@ -163,11 +226,8 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
         // Divide by two to get the number of words
         let end_address: u16 = start_address + (data.len() as u16 / 2);
         // Same as get, but a little more restrictive
-        if self.rom_range.contains(&start_address) {
-            // It's in the name, Read-Only Memory
-            Err(MockError::IllegalWriteAddress(address))
-        } else if self.ram_range.contains(&start_address) {
-            // Only the camera is allowed to write to its RAM.
+        if self.rom_range.contains(&start_address) || self.ram_range.contains(&start_address) {
+            // It's in the name, Read-Only Memory; and only the camera is allowed to write to it's RAM.
             Err(MockError::IllegalWriteAddress(address))
         } else if self.eeprom_range.contains(&start_address) {
             // Technically you can write anywhere in the EEPROM, but then the config data is lost
@@ -177,7 +237,7 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
             // anything other than 0x0000 is treated as an error.
             let slice_start = ((start_address - self.eeprom_range.start()) as usize) * 2;
             let slice_end = slice_start + data.len();
-            let existing_data = &mut self.eeprom_data[slice_start..slice_end];
+            let existing_data = &mut self.eeprom_data.borrow_mut()[slice_start..slice_end];
             // Create a mask of which bytes are writeable
             // At most 4 words will be present.
             let mut write_mask: ArrayVec<u8, { 4 * 2 }> = ArrayVec::new();
@@ -217,25 +277,35 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
             } else {
                 let new_word = u16::from_be_bytes(data.try_into().unwrap());
                 let (mask_bytes, existing_bytes) = match start_address {
-                    STATUS_REGISTER_ADDRESS => (STATUS_REGISTER_WRITE_MASK, self.status_register),
-                    CONTROL_REGISTER_ONE_ADDRESS => {
-                        (CONTROL_REGISTER_1_WRITE_MASK, self.control_register)
-                    }
-                    I2C_CONFIG_REGISTER_ADDRESS => {
-                        (I2C_CONFIG_REGISTER_WRITE_MASK, self.i2c_config_register)
+                    STATUS_REGISTER_ADDRESS => {
+                        (STATUS_REGISTER_WRITE_MASK, *self.status_register.borrow())
                     }
+                    CONTROL_REGISTER_ONE_ADDRESS => (
+                        CONTROL_REGISTER_1_WRITE_MASK,
+                        *self.control_register.borrow(),
+                    ),
+                    I2C_CONFIG_REGISTER_ADDRESS => (
+                        I2C_CONFIG_REGISTER_WRITE_MASK,
+                        *self.i2c_config_register.borrow(),
+                    ),
                     _ => return Err(MockError::IllegalWriteAddress(address)),
                 };
                 let mask_word = u16::from_be_bytes(mask_bytes);
-                let existing_word = u16::from_be_bytes(existing_bytes.try_into().unwrap());
+                let existing_word = u16::from_be_bytes(existing_bytes);
                 if !check_new_against_mask(existing_word, mask_word, new_word) {
                     let bad_address = start_address.into();
                     return Err(MockError::IllegalWriteValue(bad_address, new_word));
                 }
                 match start_address {
-                    STATUS_REGISTER_ADDRESS => self.status_register.copy_from_slice(data),
-                    CONTROL_REGISTER_ONE_ADDRESS => self.control_register.copy_from_slice(data),
-                    I2C_CONFIG_REGISTER_ADDRESS => self.i2c_config_register.copy_from_slice(data),
+                    STATUS_REGISTER_ADDRESS => {
+                        self.status_register.borrow_mut().copy_from_slice(data)
+                    }
+                    CONTROL_REGISTER_ONE_ADDRESS => {
+                        self.control_register.borrow_mut().copy_from_slice(data)
+                    }
+                    I2C_CONFIG_REGISTER_ADDRESS => {
+                        self.i2c_config_register.borrow_mut().copy_from_slice(data)
+                    }
                     _ => unreachable!(),
                 }
                 Ok(())
@@ -244,6 +314,41 @@ impl<const RAM_LENGTH: usize> MockCameraBus<RAM_LENGTH> {
             Err(MockError::UnknownMemoryAddress(address))
         }
     }
+
+    /// Replace the current RAM and status register from the given slices
+    ///
+    /// This is to simulate a new frame of data being made available. This function does *not*
+    /// explicitly set the "new data available" flag. The value in the given status register data
+    /// is used as-is.
+    pub(crate) fn update_frame(&mut self, ram_data: &[u8], status_register: &[u8]) {
+        self.ram_data.borrow_mut().copy_from_slice(ram_data);
+        self.status_register
+            .borrow_mut()
+            .copy_from_slice(status_register);
+    }
+
+    /// Set the "new data available" flag in the status register to a new value
+    pub(crate) fn set_data_available(&mut self, available: bool) {
+        if available {
+            self.status_register.borrow_mut()[1] |= 0x8;
+        } else {
+            self.status_register.borrow_mut()[1] &= 0x7;
+        }
+    }
+
+    fn add_operation(&self, operation: I2cOperation) {
+        let mut recent_ops = self.recent_operations.borrow_mut();
+        recent_ops.push_front(operation);
+        recent_ops.truncate(RECENT_OPERATIONS_QUEUE_LENGTH);
+    }
+
+    pub(crate) fn recent_operations(&self) -> Ref<VecDeque<I2cOperation>> {
+        self.recent_operations.borrow()
+    }
+
+    pub(crate) fn clear_recent_operations(&self) {
+        self.recent_operations.borrow_mut().clear()
+    }
 }
 
 impl<const RAM_LENGTH: usize> i2c::Write for MockCameraBus<RAM_LENGTH> {
@@ -257,7 +362,12 @@ impl<const RAM_LENGTH: usize> i2c::Write for MockCameraBus<RAM_LENGTH> {
         let raw_address = self.extract_address(bytes)?;
         let address: Address = raw_address.into();
         let payload = &bytes[2..];
-        self.set(address, payload)
+        self.set(address, payload)?;
+        self.add_operation(I2cOperation::Write {
+            address,
+            length: payload.len(),
+        });
+        Ok(())
     }
 }
 
@@ -274,13 +384,17 @@ impl<const RAM_LENGTH: usize> i2c::WriteRead for MockCameraBus<RAM_LENGTH> {
             return Err(MockError::UnknownI2cAddress(i2c_address));
         }
         // Write-reads should only be writing the address, so write_buffer should only be two bytes
-        if write_buffer.len() != 2 || out_buffer.len() == 0 {
+        if write_buffer.len() != 2 || out_buffer.is_empty() {
             return Err(MockError::IllegalOperation);
         }
         let raw_address = self.extract_address(write_buffer)?;
         let address: Address = raw_address.into();
+        self.add_operation(I2cOperation::Read {
+            address,
+            length: out_buffer.len(),
+        });
         let read_data = self.get(address, out_buffer.len())?;
-        out_buffer.copy_from_slice(read_data);
+        out_buffer.copy_from_slice(&read_data[..]);
         Ok(())
     }
 }
@@ -289,11 +403,11 @@ fn check_new_against_mask(existing: u16, mask: u16, new: u16) -> bool {
     (new & !mask) == (existing & !mask)
 }
 
-pub(crate) fn mock_mlx90640_at_address(i2c_address: u8) -> MockCameraBus<MLX90640_RAM_LENGTH> {
-    let eeprom_data = mlx90640_eeprom_data();
+pub(crate) fn datasheet_mlx90640_at_address(i2c_address: u8) -> MockCameraBus<MLX90640_RAM_LENGTH> {
+    let eeprom_data = mlx90640_datasheet_eeprom();
     // For the RAM, use the example data from the datasheet.
     let mut ram_data = [0u8; MLX90640_RAM_LENGTH];
-    ram_data[..(mlx90640::NUM_PIXELS * 2)]
+    ram_data[..(mlx90640::Mlx90640::NUM_PIXELS * 2)]
         .iter_mut()
         .zip(b"\x02\x61".iter().copied().cycle())
         .for_each(|(ram_byte, pixel_byte)| *ram_byte = pixel_byte);
@@ -315,27 +429,23 @@ pub(crate) fn mock_mlx90640_at_address(i2c_address: u8) -> MockCameraBus<MLX9064
     ram_data[t_a_ptat..(t_a_ptat + 2)].copy_from_slice(b"\x06\xaf");
     ram_data[cp1..(cp1 + 2)].copy_from_slice(b"\xff\xc8");
     ram_data[v_dd_pix..(v_dd_pix + 2)].copy_from_slice(b"\xcc\xc5");
-    MockCameraBus {
+    MockCameraBus::new_mlx90640(
         i2c_address,
-        rom_range: 0x0000..=0x03FF,
-        ram_range: (mlx90640::RamAddress::Base.into())..=(mlx90640::RamAddress::End.into()),
-        eeprom_range: 0x2400..=0x273F,
-        register_range: 0x8000..=0x8016,
-        eeprom_data,
-        ram_data,
+        &eeprom_data[..],
+        &ram_data[..],
         // Setting this status to mean that Subpage 1 is the current subpage, and it is new data.
-        status_register: [0x00, 0x09],
+        b"\x19\x01",
         // Default values for these taken from the datasheet
-        control_register: [0x19, 0x01],
-        i2c_config_register: [0x00, 0x00],
-    }
+        b"\x00\x09",
+        b"\x00\x00",
+    )
 }
 
 pub(crate) fn mock_mlx90641_at_address(i2c_address: u8) -> MockCameraBus<MLX90641_RAM_LENGTH> {
-    let eeprom_data = mlx90641_eeprom_data();
+    let eeprom_data = mlx90641_datasheet_eeprom();
     // For the RAM, use the example data from the datasheet.
     let mut ram_data = [0u8; MLX90641_RAM_LENGTH];
-    ram_data[..(mlx90641::NUM_PIXELS * 2)]
+    ram_data[..(mlx90641::Mlx90641::NUM_PIXELS * 2)]
         .iter_mut()
         .zip(b"\x03\xcc".iter().copied().cycle())
         .for_each(|(ram_byte, pixel_byte)| *ram_byte = pixel_byte);
@@ -364,11 +474,23 @@ pub(crate) fn mock_mlx90641_at_address(i2c_address: u8) -> MockCameraBus<MLX9064
         ram_range: (mlx90641::RamAddress::Base.into())..=(mlx90641::RamAddress::End.into()),
         eeprom_range: 0x2400..=0x273F,
         register_range: 0x8000..=0x8016,
-        eeprom_data,
-        ram_data,
+        eeprom_data: Rc::new(RefCell::new(eeprom_data)),
+        ram_data: Rc::new(RefCell::new(ram_data)),
         // The worked example uses subpage 0, so mark that as the current subpage with new data.
-        status_register: [0x00, 0x08],
-        control_register: [0x09, 0x01],
-        i2c_config_register: [0x00, 0x00],
+        status_register: Rc::new(RefCell::new([0x00, 0x08])),
+        control_register: Rc::new(RefCell::new([0x09, 0x01])),
+        i2c_config_register: Rc::new(RefCell::new([0x00, 0x00])),
+        recent_operations: Rc::new(RefCell::new(VecDeque::new())),
     }
 }
+
+pub(crate) fn example_mlx90640_at_address(i2c_address: u8) -> MockCameraBus<MLX90640_RAM_LENGTH> {
+    MockCameraBus::new_mlx90640(
+        i2c_address,
+        mlx90640_example_data::EEPROM_DATA,
+        mlx90640_example_data::FRAME_0_DATA,
+        mlx90640_example_data::CONTROL_REGISTER,
+        mlx90640_example_data::FRAME_0_STATUS_REGISTER,
+        b"\x00\x00",
+    )
+}
